 
;
;
; Sluzba - Precteni pozadovaneho sektoru
;
; Vstup:
;
;    FDC_DRIVE_ID,                       /* id zvolene mechaniky 0 - 3 */
;    FDC_DRIVE_PARAM,                    /* parametry media */
;    unsigned char log_track,            /* logicka stopa */ 
;    unsigned char phy_sector,           /* fyzicky sektor */
;    void* dma                           /* adresa dat */
;
;
; Vystup:
;		 0: OK
;		-1: ERROR
;
_FDC_ReadSector::
	push ix
	ld ix,#0x0004			; nastavime index 0 na 1 parametr funkce
	add ix, sp

.if MULTITASK_OS
	di					; SYSTEM_DI_REQUEST
.endif

	call __FDC_ReadSector_routine

.if MULTITASK_OS
	ei					; SYSTEM_EI_REQUEST
.endif

	pop ix
	ret


;
; Spolecna rutina pro cteni sektoru
;
__FDC_ReadSector_routine::

	call __FDC_PrepareSectorSizeAndCpl

	ld (#__FDC_ReadData_sector_size), hl	; nastavime velikost sektoru

	ld a,c
	ld (#__FDC_ReadData_cpl), a		; upravime DRQ rutinu

	
	call __FDC_driveON
	ld b,#0x05				; max pocet pokusu

__FDC_Read_nextAttempt:	
	push bc

	call __FDC_setSectorSideTrack		; nastav stopu, sector a stranu

	jr nc, __FDC_serviceRead_error		; Nastaveni se nezdarilo?

	call __FDC_DI_createSideFromTrack	; Zakazeme preruseni a do A pripravime overeni strany

	xor #__FDC_CMD_READ_SECTOR|__FDC_TYPII_SINGLE_SEC|__FDC_TYPII_NO_DLY_15MS|__FDC_TYPII_VRF_SIDE|__FDC_TYPII_DATA_SIG_FB|0x10

	out (__FDC_CMD),a

	call __FDC_waitForNotBusy
	call __FDC_ReadData			; Cteme data, pokud je vse OK, tak Carry = 1


.if MULTITASK_OS
.else
	ei					; SYSTEM_EI_REQUEST
.endif

	jr nc, __FDC_serviceRead_error		; Cteni bylo ukonceno s chybou?
	call __FDC_waitForCmdDone

	; zaverecne overeni statusu
	in a,(__FDC_STS)
	cpl
	and #0xbf				; maska podstatnych status bitu po READ_SECTOR
	scf					; pokud je to OK, tak Carry = 1
	jr z, __FDC_serviceReadWrite_done	; Status bez chyby?


__FDC_serviceRead_error:
	call __FDC_VerifyTrackByAddr		; overime format diskety a stopu podle ADDR

	pop bc
	djnz __FDC_Read_nextAttempt		; zkusime to cteni zopakovat 5x

	; cp/m tady ma volani RETRY/ABORT/IGNORE
	; v pripade, ze chceme proces zopakovat,
	; tak hupsneme zpet na _FDC_serviceRead

	ld l,#0xff				; return ( -1 )
	ret

__FDC_serviceReadWrite_done:
	pop bc
	ld l,a					; return ( A )
	ret




;
;
; Sluzba - Zapis pozadovaneho sektoru
;
; Vstup:
;
;    FDC_DRIVE_ID,                       /* id zvolene mechaniky 0 - 3 */
;    FDC_DRIVE_PARAM,                    /* parametry media */
;    unsigned char log_track,            /* logicka stopa */ 
;    unsigned char phy_sector,           /* fyzicky sektor */
;    void* dma                           /* adresa dat */
;
;
; Vystup:
;		 0: OK
;		-1: ERROR
;		-2: WR_PROTECT
;
_FDC_WriteSector::
	push ix
	ld ix,#0x0004			; nastavime index 0 na 1 parametr funkce
	add ix, sp

.if MULTITASK_OS
	di					; SYSTEM_DI_REQUEST
.endif

	call __FDC_WriteSector_routine

.if MULTITASK_OS
	ei					; SYSTEM_EI_REQUEST
.endif

	pop ix
	ret

 
__FDC_WriteSector_routine::

	call __FDC_PrepareSectorSizeAndCpl

	; u zapisu potrebujeme sector size prevest na skutecnou hodnotu
	ld a,l
	ld b,h
	bit 7,b
	jr z,1$

	xor a					; u 128 B nulujeme horni bajt 

	1$:
	ld h,a
	ld l,b

	ld (#__FDC_WriteData_sector_size), hl	; nastavime velikost sektoru

	ld a,c
	ld (#__FDC_WriteData_cpl), a		; upravime DRQ rutinu

	
	call __FDC_driveON
	ld b,#0x05				; max pocet pokusu

__FDC_Write_nextAttempt:	
	push bc

	call __FDC_setSectorSideTrack		; nastav stopu, sector a stranu

	jr nc, __FDC_serviceWrite_error		; Nastaveni se nezdarilo?

	
	; udelame test na WR_PRT
	call __FDC_SendBreakWhileIsBusy
	in a, (__FDC_STS)
	bit 6,a
	ld a,#0xfe
	jr z,__FDC_serviceReadWrite_done	; WR_PRT? return ( -2 )
	call __FDC_delay90us

	call __FDC_DI_createSideFromTrack	; Zakazeme preruseni a do A pripravime overeni strany

	xor #__FDC_CMD_WRITE_SECTOR|__FDC_TYPII_SINGLE_SEC|__FDC_TYPII_NO_DLY_15MS|__FDC_TYPII_VRF_SIDE|__FDC_TYPII_DATA_SIG_FB|0x10

	out (__FDC_CMD),a

	call __FDC_waitForNotBusy
	call __FDC_WriteData			; Dokud je BUSY, tak se pokousime zapisovat data


.if MULTITASK_OS
.else
	ei					; SYSTEM_EI_REQUEST
.endif


	ld a,#0x03
	call __FDC_delay1MSxA	

	; prenesli jsme ocekavany pocet bajtu?
	.db 0x01				; LD BC,nm
	__FDC_WriteData_sector_size:
	.dw 0x0000

	ld l,4 (ix)				; __FDC_Requested_dma
	ld h,5 (ix)				; do HL

	add hl,bc
	dec de
	scf
	ccf
	sbc hl,de
	jr nz, __FDC_serviceWrite_error		; pocet zapsanych bajtu nesouhlasi s velikosti sektoru!

	; zaverecne overeni statusu
	in a,(__FDC_STS)
	inc a
	scf					; pokud je vse OK, tak vratime Carry = 1
	jr z, __FDC_serviceReadWrite_done	; Status bez chyby? return ( 0 )


__FDC_serviceWrite_error:
	call __FDC_VerifyTrackByAddr		; overime format diskety a stopu podle ADDR

	pop bc
	djnz __FDC_Write_nextAttempt		; zkusime ten zapis zopakovat 5x

	; cp/m tady ma volani RETRY/ABORT/IGNORE
	; v pripade, ze chceme proces zopakovat,
	; tak hupsneme zpet na _FDC_serviceWrite

	ld l,#0xff				; return ( -1 )
	ret



;
; Dokud je BUSY, tak cekame na DRQ - kdyz prijde, tak zapiseme pripraveny bajt.
;
; Pokud zmizi BUSY, tak se vratime s Carry = 1 a rotovanym statusem v A
;
__FDC_WriteData::
	ld c,#__FDC_DATA

	ld e,4 (ix)		; __FDC_Requested_dma
	ld d,5 (ix)		; do DE

	ld hl,#__FDC_WriteData_prepare_byte

__FDC_WriteData_prepare_byte:
	ld a,(de)		; precti bajt z DMA
__FDC_WriteData_cpl:
	.db 0x00		; cpl, nebo nop
	inc de
	ld b,a			; do B pripraveny bajt

	1$:
	in a,(__FDC_STS)
	rra
	ret c			; uz neni BUSY, Carry = 1
	rra
	jr c,1$			; cekame na DRQ

	out (c),b		; zapis bajt
	jp (hl)			; jp (hl) ma jen 1 bajt a spotrebuje 4T


;
; Pokud prijde DRQ, tak precteme pozadovany pocet bajtu.
;
; Vraci:
;		Carry = 1: OK
;		Carry = 0: ERROR - timeout
;
__FDC_ReadData::
	.db 01			; LD BC,nm
__FDC_ReadData_sector_size:
	.dw 0x0002		; sector size
	ld de,#0x0000		; error timer

	ld l,4 (ix)		; __FDC_Requested_dma
	ld h,5 (ix)		; do HL

	; cekame na data
	__FDC_ReadData_dec_timer:
	dec de
	ld a,d
	or e
	ret z			; timeout, Carry = 0

	__FDC_ReadData_read_sts:
	in a,(__FDC_STS)
	and #0x02
	jr nz,__FDC_ReadData_dec_timer		; cekame na DRQ

	; DRQ rutina
	in a,(__FDC_DATA)

__FDC_ReadData_cpl:
	.db 0x00		; nop, nebo cpl

	ld (hl),a

	inc hl
	djnz __FDC_ReadData_read_sts
	dec c
	jp nz,__FDC_ReadData_read_sts

	scf			; OK, Carry = 1
	ret
